package com.ippon.pouet.common.infrastructure.primary;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
import com.ippon.pouet.common.infrastructure.primary.PouetFieldError.PouetFieldErrorBuilder;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

@JsonDeserialize(builder = PouetFieldErrorBuilder.class)
@ApiModel(description = "Error for a field validation")
class PouetFieldError {
  @ApiModelProperty(value = "Path to the field in error", example = "address.country", required = true)
  private final String fieldPath;

  @ApiModelProperty(value = "Technical reason for the invalidation", example = "user.mandatory", required = true)
  private final String reason;

  @ApiModelProperty(value = "Human readable message for the invalidation", example = "Le champ doit être renseigné", required = true)
  private final String message;

  private PouetFieldError(PouetFieldErrorBuilder builder) {
    fieldPath = builder.fieldPath;
    reason = builder.reason;
    message = builder.message;
  }

  public static PouetFieldErrorBuilder builder() {
    return new PouetFieldErrorBuilder();
  }

  public String getFieldPath() {
    return fieldPath;
  }

  public String getReason() {
    return reason;
  }

  public String getMessage() {
    return message;
  }

  @JsonPOJOBuilder(withPrefix = "")
  public static class PouetFieldErrorBuilder {
    private String fieldPath;
    private String reason;
    private String message;

    public PouetFieldErrorBuilder fieldPath(String fieldPath) {
      this.fieldPath = fieldPath;

      return this;
    }

    public PouetFieldErrorBuilder reason(String reason) {
      this.reason = reason;

      return this;
    }

    public PouetFieldErrorBuilder message(String message) {
      this.message = message;

      return this;
    }

    public PouetFieldError build() {
      return new PouetFieldError(this);
    }
  }
}
